import os
import shutil
import subprocess
import sys
import tempfile
from pathlib import Path

import ida_domain.entries
import pytest

import ida_domain  # isort: skip
import ida_idaapi  # isort: skip

idb_path: str = ''


# Global setup (runs ONCE)
@pytest.fixture(scope='module', autouse=True)
def global_setup():
    print(f'\nAPI Version: {ida_domain.__version__}')

    os.environ['IDA_NO_HISTORY'] = '1'

    """ Runs once per module: Creates temp directory and writes test binary. """
    global idb_path
    # Create a temporary folder and use it as tests working directory
    idb_path = os.path.join(tempfile.gettempdir(), 'api_tests_work_dir')
    shutil.rmtree(idb_path, ignore_errors=True)
    os.makedirs(idb_path, exist_ok=True)
    idb_path = os.path.join(tempfile.gettempdir(), 'api_tests_work_dir', 'test.bin')

    # Copy the test binary from resources folder under our tests working directory
    current_dir = os.path.dirname(os.path.abspath(__file__))
    src_path = os.path.join(current_dir, 'resources', 'test.bin')
    shutil.copy(src_path, idb_path)


# Per-test fixture (runs for each test)
@pytest.fixture(scope='function')
def test_env():
    """Runs for each test: Opens and closes the database."""
    db = ida_domain.Database()
    assert db.open(idb_path)
    yield db
    db.close(False)


def test_database(test_env):
    db = test_env
    db.close(False)
    assert db.is_open() is False
    assert db.open(idb_path) is True
    assert db.is_open() is True

    assert db.entry_point == 0

    db.current_ea = db.entry_point
    assert db.current_ea == db.entry_point

    assert db.minimum_ea == 0x0
    assert db.maximum_ea == 0x3F0

    assert db.base_address == 0x0
    assert db.module == 'test.bin'
    assert db.filesize == 3488
    assert db.md5 == 'd87c1a6b9ce33ee84f9ed41b6bc5c2e2'
    assert db.sha256 == '21301dd089510325aeda4869a57dfe9348b53a760978ca5755ade92edddeb1d6'
    assert db.crc32 == 321919723
    assert db.architecture == 'metapc'
    assert db.bitness == 64
    assert db.format == 'ELF64 for x86-64 (Relocatable)'

    metadata = db.metadata
    assert len(metadata) == 11

    assert 'path' in metadata
    assert 'test.bin' in metadata['path']

    assert metadata['module'] == 'test.bin'
    assert metadata['base_address'] == '0x0'
    assert metadata['filesize'] == '0xda0'
    assert metadata['md5'] == 'd87c1a6b9ce33ee84f9ed41b6bc5c2e2'
    assert metadata['sha256'] == '21301dd089510325aeda4869a57dfe9348b53a760978ca5755ade92edddeb1d6'
    assert metadata['crc32'] == '0x13301aeb'
    assert metadata['architecture'] == 'metapc'
    assert metadata['bitness'] == '0x40'
    assert metadata['format'] == 'ELF64 for x86-64 (Relocatable)'

    assert 'load_time' in metadata
    assert isinstance(metadata['load_time'], str)
    assert len(metadata['load_time']) == 19  # dummy check, expect "YYYY-MM-DD HH:MM:SS"


def test_segment(test_env):
    db = test_env
    all_segments = db.segments.get_all()
    seg = next(all_segments)
    assert seg is not None
    assert db.segments.get_name(seg) == '.text'
    assert seg.start_ea == 0
    assert db.segments.set_name(seg, 'testing_segment_rename')
    assert db.segments.get_name(seg) == 'testing_segment_rename'

    seg = next(all_segments)
    assert seg is not None
    assert db.segments.get_name(seg) == '.data'
    assert seg.start_ea == 0x300


def test_function(test_env):
    db = test_env

    all_functions = db.functions.get_all()
    func = next(all_functions)
    assert func is not None
    assert db.functions.get_name(func) == 'test_all_operand_types'

    func = next(all_functions)
    assert func is not None
    assert db.functions.get_name(func) == 'add_numbers'
    assert func.start_ea == 0x29E

    func = db.functions.get_at(0x29E)
    assert func is not None
    assert func.start_ea == 0x29E
    assert db.functions.set_name(func, 'testing_function_rename')
    assert db.functions.get_name(func) == 'testing_function_rename'
    assert db.functions.set_name(func, 'add_numbers')
    assert db.functions.get_name(func) == 'add_numbers'

    blocks = db.functions.get_basic_blocks(func)
    assert blocks.size == 1
    assert blocks[0].start_ea == 0x29E
    assert blocks[0].end_ea == 0x2AA

    disassembly_lines = db.functions.get_disassembly(func)
    assert len(disassembly_lines) == 6

    pseudocode_lines = db.functions.get_pseudocode(func)
    assert len(pseudocode_lines) == 4

    microcode_lines = db.functions.get_microcode(func)
    assert len(microcode_lines) == 13
    assert microcode_lines[11] == '1.11 mov    cs.2, seg.2             ; 2A9 u=cs.2       d=seg.2'

    # Validate expected instructions and their addresses
    expected_instructions = [
        (0x29E, 'push    rbp'),
        (0x29F, 'mov     rbp, rsp'),
        (0x2A2, 'mov     rax, rdi'),
        (0x2A5, 'add     rax, rsi'),
        (0x2A8, 'pop     rbp'),
        (0x2A9, 'retn'),
        (ida_idaapi.BADADDR, ''),
    ]

    instructions = db.functions.get_instructions(func)
    for i, instruction in enumerate(instructions):
        assert expected_instructions[i][0] == instruction.ea
        assert expected_instructions[i][1] == db.instructions.get_disassembly(instruction)

    func = db.functions.get_at(0x29E)
    assert func is not None

    # Validate function signature
    expected_signature = '__int64 __fastcall(__int64, __int64)'
    assert db.functions.get_signature(func) == expected_signature

    # Check signature match
    assert db.functions.matches_signature(func, expected_signature)
    assert not db.functions.matches_signature(func, 'void __fastcall()')

    # Remove and re-create function
    assert db.functions.remove(0xC6)
    assert db.functions.get_at(0xC6) is None

    assert db.functions.create(0xC6)
    assert db.functions.get_at(0xC6) is not None


def test_entries(test_env):
    db = test_env

    count = 0
    entries = db.entries.get_all()
    for _ in entries:
        count += 1
    assert count == 1

    assert db.entries.get_count() == 1
    assert db.entries.get_at_index(0) == ida_domain.entries.Entry(0, 0, '_start', None)
    assert db.entries.get_by_ordinal(0) == ida_domain.entries.Entry(0, 0, '_start', None)
    assert db.entries.get_by_address(0) == ida_domain.entries.Entry(0, 0, '_start', None)

    assert db.entries.add(address=0xCC, name='test_entry', ordinal=1)
    assert db.entries.get_count() == 2
    assert db.entries.get_at_index(1) == ida_domain.entries.Entry(1, 0xCC, 'test_entry', None)
    assert db.entries.get_by_ordinal(1) == ida_domain.entries.Entry(1, 0xCC, 'test_entry', None)
    assert db.entries.get_by_address(0xCC) == ida_domain.entries.Entry(1, 0xCC, 'test_entry', None)

    assert db.entries.rename(0, '_new_start')
    assert db.entries.get_at_index(0) == ida_domain.entries.Entry(0, 0, '_new_start', None)

    assert db.entries.get_by_name('_new_start') == ida_domain.entries.Entry(
        0, 0, '_new_start', None
    )


def test_heads(test_env):
    db = test_env

    count = 0
    heads = db.heads.get_all()
    for _ in heads:
        count += 1
    assert count == 180

    assert db.heads.get_prev(db.minimum_ea) is None
    assert db.heads.get_next(db.maximum_ea) is None

    expected = [0xC6, 0xC8, 0xCA, 0xCC, 0xCF, 0xD1, 0xD4]
    actual = []
    heads = db.heads.get_between(0xC6, 0xD6)
    for ea in heads:
        actual.append(ea)
    assert actual == expected

    assert db.heads.get_prev(0xCA) == 0xC8
    assert db.heads.get_next(0xC8) == 0xCA


def test_instruction(test_env):
    db = test_env

    count = 0
    for instruction in db.instructions.get_between(db.minimum_ea, db.maximum_ea):
        count += 1
    assert count == 176

    instruction = db.instructions.get_at(0xD6)
    assert instruction is not None
    assert db.instructions.is_valid(instruction)
    assert db.instructions.get_disassembly(instruction) == 'movzx   rax, ax'
    assert db.instructions.get_operands_count(instruction) == 2

    operands = db.instructions.get_operands(instruction)
    assert len(operands) == 2
    assert isinstance(operands[0], ida_domain.operands.RegisterOperand)
    assert isinstance(operands[1], ida_domain.operands.RegisterOperand)

    operands = db.instructions.get_operands(instruction)
    assert len(operands) == 2
    assert isinstance(operands[0], ida_domain.operands.RegisterOperand)
    assert isinstance(operands[1], ida_domain.operands.RegisterOperand)

    instruction = db.instructions.get_prev(0xD6)
    assert instruction is not None
    assert instruction.ea == 0xD4
    assert db.instructions.is_valid(instruction)
    assert db.instructions.get_disassembly(instruction) == 'mov     al, bl'
    assert db.instructions.get_operands_count(instruction) == 2

    operands = db.instructions.get_operands(instruction)
    assert len(operands) == 2
    assert isinstance(operands[0], ida_domain.operands.RegisterOperand)
    assert isinstance(operands[1], ida_domain.operands.RegisterOperand)


def test_basic_block(test_env):
    db = test_env
    func = db.functions.get_at(0x29E)
    assert func is not None

    blocks = db.functions.get_basic_blocks(func)
    assert blocks.size == 1

    # Validate expected blocks
    expected_blocks = [(0x29E, 0x2AA)]

    for i, block in enumerate(blocks):
        assert expected_blocks[i][0] == block.start_ea, (
            f'Block start ea mismatch at index {i}, '
            f'{hex(expected_blocks[i][0])} != {hex(block.start_ea)}'
        )
        assert expected_blocks[i][1] == block.end_ea, (
            f'Block end ea mismatch at index {i}, '
            f'{hex(expected_blocks[i][1])} != {hex(block.end_ea)}'
        )

    # Validate expected instructions and their addresses
    expected_instructions = [
        (0x29E, 'push    rbp'),
        (0x29F, 'mov     rbp, rsp'),
        (0x2A2, 'mov     rax, rdi'),
        (0x2A5, 'add     rax, rsi'),
        (0x2A8, 'pop     rbp'),
        (0x2A9, 'retn'),
        (ida_idaapi.BADADDR, ''),
    ]

    instructions = db.basic_blocks.get_instructions(blocks[0])
    for i, instruction in enumerate(instructions):
        assert expected_instructions[i][0] == instruction.ea
        assert expected_instructions[i][1] == db.instructions.get_disassembly(instruction)


def test_operands_comprehensive(test_env):
    db = test_env

    # Test basic register operand - mov rax, rdi at 0x2A2
    instruction = db.instructions.get_at(0x2A2)
    operands = db.instructions.get_operands(instruction)

    # First operand should be rax (destination register)
    reg_op = operands[0]
    assert isinstance(reg_op, ida_domain.operands.RegisterOperand)
    assert reg_op.get_type_name() == 'register'
    assert reg_op.get_register_name() == 'rax'
    assert reg_op.register_number == 0  # rax register number
    assert reg_op.get_access_type() == 'write'
    assert reg_op.is_write() and not reg_op.is_read()

    # Test base operand info
    base_info = reg_op.get_info()
    assert base_info.number == 0
    assert base_info.type_name == 'register'
    assert base_info.access_type == 'write'
    assert base_info.is_write
    assert not base_info.is_read

    # Second operand should be rdi (source register)
    reg_op2 = operands[1]
    assert isinstance(reg_op2, ida_domain.operands.RegisterOperand)
    assert reg_op2.get_register_name() == 'rdi'
    assert reg_op2.get_access_type() == 'read'
    assert reg_op2.is_read() and not reg_op2.is_write()

    # Test immediate value - mov edi, 1 at 0x5
    instruction = db.instructions.get_at(0x5)
    operands = db.instructions.get_operands(instruction)

    imm_op = operands[1]  # Second operand should be immediate 1
    assert isinstance(imm_op, ida_domain.operands.ImmediateOperand)
    assert imm_op.get_type_name() == 'immediate'
    assert imm_op.get_value() == 1
    assert imm_op.get_hex_value() == '0x1'  # Small values stay decimal
    assert not imm_op.is_address()
    assert imm_op.get_name() is None  # Not an address

    # Test immediate info dataclass
    imm_info = imm_op.get_immediate_info()
    assert imm_info.value == 1
    assert imm_info.hex_value == '0x1'
    assert not imm_info.is_address
    assert imm_info.symbol_name is None

    # Test larger immediate value - mov rax, 1234567890ABCDEFh at 0xDD
    instruction = db.instructions.get_at(0xDD)
    operands = db.instructions.get_operands(instruction)

    large_imm_op = operands[1]
    assert isinstance(large_imm_op, ida_domain.operands.ImmediateOperand)
    large_value = large_imm_op.get_value()
    assert large_value == 0x1234567890ABCDEF
    hex_val = large_imm_op.get_hex_value()
    assert hex_val.startswith('0x')

    # Test Near Address Operands (calls/jumps)
    # Find a call instruction - call add_numbers at 0x27
    instruction = db.instructions.get_at(0x27)
    operands = db.instructions.get_operands(instruction)

    addr_op = operands[0]
    assert isinstance(addr_op, ida_domain.operands.ImmediateOperand)
    assert addr_op.get_type_name() == 'near_address'
    assert addr_op.is_address()
    symbol_name = addr_op.get_name()
    assert symbol_name == 'add_numbers'  # Should resolve to function name

    # Test address info
    addr_info = addr_op.get_immediate_info()
    assert addr_info.is_address
    assert addr_info.symbol_name == 'add_numbers'

    # Test direct memory access - mov rax, test_data at 0xFA
    instruction = db.instructions.get_at(0xFA)
    operands = db.instructions.get_operands(instruction)

    mem_op = operands[1]
    assert isinstance(mem_op, ida_domain.operands.MemoryOperand)
    assert mem_op.get_type_name() == 'direct_memory'
    assert mem_op.is_direct_memory()
    assert not mem_op.is_register_based()
    assert mem_op.get_sib_components() == (None, None, None, None)

    # Test memory address and symbol
    addr = mem_op.get_address()
    assert addr is not None
    symbol = mem_op.get_name()
    assert symbol == 'test_data'

    # Test addressing info dataclass
    addr_info = mem_op.get_addressing_info()
    assert addr_info.addressing_type == 'direct_memory'
    assert addr_info.is_direct_memory()
    assert addr_info.address == addr
    assert addr_info.symbol_name == 'test_data'

    # Test register indirect - mov rax, [rbx] at 0x120
    instruction = db.instructions.get_at(0x120)
    operands = db.instructions.get_operands(instruction)

    phrase_op = operands[1]
    assert isinstance(phrase_op, ida_domain.operands.MemoryOperand)
    assert phrase_op.get_type_name() == 'register_indirect'
    assert phrase_op.is_register_based()
    assert not phrase_op.is_direct_memory()
    assert phrase_op.get_sib_components() == ('rbx', None, None, None)

    # Test phrase number
    phrase_num = phrase_op.get_phrase_number()
    assert phrase_num is not None

    # Test formatted string
    formatted = phrase_op.get_formatted_string()
    assert '[rbx]' in formatted

    # Test addressing info
    addr_info = phrase_op.get_addressing_info()
    assert addr_info.addressing_type == 'register_indirect'
    assert addr_info.is_register_based()
    assert addr_info.phrase_number == phrase_num
    assert addr_info.formatted_string is not None

    # Test register+displacement - mov rax, [rbp+8] at 0x128
    instruction = db.instructions.get_at(0x128)
    operands = db.instructions.get_operands(instruction)

    disp_op = operands[1]
    assert isinstance(disp_op, ida_domain.operands.MemoryOperand)
    assert disp_op.get_type_name() == 'register_displacement'
    assert disp_op.is_register_based()
    assert disp_op.get_sib_components() == ('rbp', None, None, 8)

    # Test displacement value
    displacement = disp_op.get_displacement()
    assert displacement is not None
    assert displacement == 8  # [rbp+8]

    # Test outer displacement (should be None for simple displacement)
    outer_disp = disp_op.get_outer_displacement()
    assert outer_disp is None

    # Test has_outer_displacement flag
    assert not disp_op.has_outer_displacement()

    # Test addressing info
    addr_info = disp_op.get_addressing_info()
    assert addr_info.addressing_type == 'register_displacement'
    assert addr_info.displacement == 8
    assert not addr_info.has_outer_displacement
    assert addr_info.outer_displacement is None

    formatted = disp_op.get_formatted_string()
    assert '[rbp+' in formatted and '8' in formatted

    # Test complex displacement - mov rax, [rsi+rdi*2+8] at 0x15d
    instruction = db.instructions.get_at(0x15D)
    operands = db.instructions.get_operands(instruction)

    complex_disp_op = operands[1]
    assert isinstance(complex_disp_op, ida_domain.operands.MemoryOperand)
    assert complex_disp_op.get_type_name() == 'register_displacement'

    formatted = complex_disp_op.get_formatted_string()
    assert 'rsi' in formatted and 'rdi' in formatted and '*2' in formatted

    # Test Operand Value Method Consistency
    # Register operand value should be register number
    reg_val = reg_op.get_value()
    assert isinstance(reg_val, int)

    # Memory operand values vary by type
    mem_val = complex_disp_op.get_value()
    assert isinstance(mem_val, dict)  # Displacement operands return dict
    assert 'phrase' in mem_val and 'displacement' in mem_val

    # All operands should have meaningful string representations
    reg_str = str(reg_op)
    assert 'RegisterOperand' in reg_str
    assert 'Op0' in reg_str  # Operand number

    mem_str = str(mem_op)
    assert 'MemoryOperand' in mem_str
    assert complex_disp_op.get_sib_components() == ('rsi', 'rdi', 2, 8)


def test_strings(test_env):
    db = test_env

    assert db.strings.get_count() == 3

    expected_strings = [
        (0x370, 'Source string data'),
        (0x3A4, 'Hello, IDA!\n'),
        (0x3B1, 'Sum: Product: \n'),
    ]

    for i, (expected_addr, expected_string) in enumerate(expected_strings):
        stringsAndAddress = db.strings.get_at_index(i)
        assert stringsAndAddress[0] == expected_addr
        assert stringsAndAddress[1] == expected_string

    for i, (addr, string) in enumerate(db.strings.get_all()):
        assert addr == expected_strings[i][0], (
            f'String address mismatch at index {i}, {hex(addr)} != {hex(expected_strings[i][0])}'
        )
        assert string == expected_strings[i][1], (
            f'String mismatch at index {i}, {string} != {expected_strings[i][1]}'
        )

    string = db.strings.get_at(0x3A4)
    assert string == 'Hello, IDA!\n'

    string = db.strings.get_at(0x3B1)
    assert string == 'Sum: Product: \n'


def test_names(test_env):
    db = test_env

    assert db.names.get_count() == 24

    expected_names = [
        (0x0, '_start'),
        (0xBF, 'test_all_operand_types'),
        (0x26D, 'skip_jumps'),
        (0x29E, 'add_numbers'),
        (0x2AA, 'multiply_numbers'),
        (0x2B7, 'print_number'),
        (0x2CB, 'print_number.print_digit'),
        (0x300, 'test_data'),
        (0x308, 'test_array'),
        (0x348, 'temp_float'),
        (0x34C, 'temp_double'),
        (0x360, 'vector_data'),
        (0x370, 'src_string'),
        (0x383, 'dst_string'),
        (0x3A4, 'hello'),
        (0x3B1, 'sum_str'),
        (0x3B6, 'product_str'),
        (0x3BF, 'newline'),
        (0x3C0, 'float_val'),
        (0x3C4, 'double_val'),
        (0x3D0, 'hello_len'),
        (0x3D8, 'sum_len'),
        (0x3E0, 'product_len'),
        (0x3E8, 'newline_len'),
    ]

    for i, (expected_addr, expected_name) in enumerate(expected_names):
        nameAndAddress = db.names.get_at_index(i)
        assert nameAndAddress[0] == expected_addr, (
            f'Name address mismatch at index {i}, {hex(nameAndAddress[0])} != {hex(expected_addr)}'
        )
        assert nameAndAddress[1] == expected_name, (
            f'Name mismatch at index {i}, {nameAndAddress[1]} != {expected_name}'
        )

    all_names = db.names.get_all()
    for i, (addr, name) in enumerate(all_names):
        assert addr == expected_names[i][0]
        assert name == expected_names[i][1]

    name = db.names.get_at(0x0)
    assert name == '_start'

    name = db.names.get_at(0x3E8)
    assert name == 'newline_len'

    assert db.names.get_at(db.entry_point) == '_start'


def test_xrefs(test_env):
    db = test_env
    expected_xrefs = [0xC4]
    expected_names = ['Ordinary_Flow']
    xrefs_to = db.xrefs.get_to(0xC6)
    for i, xrefblk in enumerate(xrefs_to):
        assert xrefblk.frm == expected_xrefs[i]
        assert db.xrefs.get_name(xrefblk) == expected_names[i]

    expected_xrefs = [0xDA]
    expected_names = ['Ordinary_Flow']
    xrefs_from = db.xrefs.get_from(0xD6)
    for i, xrefblk in enumerate(xrefs_from):
        assert xrefblk.to == expected_xrefs[i]
        assert db.xrefs.get_name(xrefblk) == expected_names[i]


def test_types(test_env):
    db = test_env
    all_types = db.types.get_names()
    count = 0
    for _ in all_types:
        count = count + 1
    assert count == 0

    type_name = db.types.get_name_at(0xB3)
    assert type_name is None

    assert not db.types.apply_named_type(0xB3, 'int')
    type_name = db.types.get_name_at(0xB3)
    assert type_name is None


def test_flirt(test_env):
    db = test_env

    # Get available signatures
    available_sigs = db.flirt.get_files()
    assert len(available_sigs) > 0, 'No signature files found'


def test_comments(test_env):
    db = test_env

    # Validate expected comments and their addresses
    expected_comments = [
        (0x16, 'LINUX - sys_write'),
        (0x46, 'LINUX - sys_write'),
        (0x67, 'LINUX - sys_write'),
        (0x92, 'LINUX - sys_write'),
        (0xB3, 'LINUX - sys_write'),
        (0xBD, 'LINUX - sys_exit'),
        (0x2D1, 'buf'),
        (0x2E0, 'fd'),
        (0x2E8, 'count'),
        (0x2EB, 'LINUX - sys_write'),
    ]

    for i, (addr, comment) in enumerate(db.comments.get_all(False)):
        assert expected_comments[i][0] == addr
        assert expected_comments[i][1] == comment

    # Add a new comment at address 0xae
    assert db.comments.set(0xAE, 'Testing adding comment')
    assert db.comments.get(0xAE) == 'Testing adding comment'


def test_bytes(test_env):
    db = test_env

    byte_val = db.bytes.get_byte(0x3CA)
    assert byte_val == 0x19

    word_val = db.bytes.get_word(0x3C0)
    assert word_val == 0xF5C3

    dword_val = db.bytes.get_dword(0x3B8)
    assert dword_val == 0x6375646F

    qword_val = db.bytes.get_qword(0x3BD)
    assert qword_val == 0x1F4048F5C30A203A

    float_val = db.bytes.get_float(0x12C)
    assert pytest.approx(float_val, rel=3.14) == 0.0

    double_val = db.bytes.get_double(0x130)
    assert pytest.approx(double_val, rel=6.28) == 0.0

    disasm = db.bytes.get_disassembly(0x3A4)
    assert disasm == "db 'Hello, IDA!',0Ah,0"


def test_ida_command_builder_default_state():
    """Test default state produces empty args"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.build_args() == ''


def test_ida_command_builder_auto_analysis():
    """Test auto analysis option"""
    builder = ida_domain.IdaCommandBuilder()
    # Default is true, should only add flag if false
    assert builder.auto_analysis(True).build_args() == ''

    builder = ida_domain.IdaCommandBuilder()
    assert builder.auto_analysis(False).build_args() == '-a'


def test_ida_command_builder_autonomous():
    """Test autonomous mode option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.autonomous().build_args() == '-A'

    builder = ida_domain.IdaCommandBuilder()
    assert builder.autonomous(False).build_args() == ''


def test_ida_command_builder_loading_address():
    """Test loading address option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_loading_address(0x1000).build_args() == '-b1000'


def test_ida_command_builder_new_database():
    """Test new database option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.new_database().build_args() == '-c'


def test_ida_command_builder_compiler():
    """Test compiler option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_compiler('gcc').build_args() == '-Cgcc'

    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_compiler('gcc', 'x64').build_args() == '-Cgcc:x64'


def test_ida_command_builder_first_pass_directive():
    """Test first pass directive option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.add_first_pass_directive('VPAGESIZE=8192').build_args() == '-dVPAGESIZE=8192'

    # Add multiple directives
    builder = ida_domain.IdaCommandBuilder()
    builder.add_first_pass_directive('DIR1')
    builder.add_first_pass_directive('DIR2')
    assert builder.build_args() == '-dDIR1 -dDIR2'


def test_ida_command_builder_second_pass_directive():
    """Test second pass directive option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.add_second_pass_directive('OPTION=VALUE').build_args() == '-DOPTION=VALUE'


def test_ida_command_builder_disable_fpp():
    """Test disable FPP instructions option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.disable_fpp_instructions().build_args() == '-f'


def test_ida_command_builder_entry_point():
    """Test entry point option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_entry_point(0x401000).build_args() == '-i401000'


def test_ida_command_builder_jit_debugger():
    """Test JIT debugger option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_jit_debugger(True).build_args() == '-I1'

    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_jit_debugger(False).build_args() == '-I0'


def test_ida_command_builder_log_file():
    """Test log file option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_log_file('debug.log').build_args() == '-Ldebug.log'


def test_ida_command_builder_disable_mouse():
    """Test disable mouse option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.disable_mouse().build_args() == '-M'


def test_ida_command_builder_plugin_options():
    """Test plugin options"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_plugin_options('opt1=val1').build_args() == '-Oopt1=val1'


def test_ida_command_builder_output_database():
    """Test output database option (should also set -c flag)"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_output_database('output.idb').build_args() == '-c -ooutput.idb'


def test_ida_command_builder_processor():
    """Test processor option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_processor('arm').build_args() == '-parm'


def test_ida_command_builder_database_compression():
    """Test database compression options"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.compress_database().build_args() == '-P+'

    builder = ida_domain.IdaCommandBuilder()
    assert builder.pack_database().build_args() == '-P'

    builder = ida_domain.IdaCommandBuilder()
    assert builder.no_pack_database().build_args() == '-P-'


def test_ida_command_builder_run_debugger():
    """Test run debugger option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.run_debugger().build_args() == '-r'

    builder = ida_domain.IdaCommandBuilder()
    assert builder.run_debugger('debug-options').build_args() == '-rdebug-options'


def test_ida_command_builder_load_resources():
    """Test load resources option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.load_resources().build_args() == '-R'


def test_ida_command_builder_run_script():
    """Test run script option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.run_script('analyze.py').build_args() == '-Sanalyze.py'

    builder = ida_domain.IdaCommandBuilder()
    args = ['arg1', 'arg with spaces', '--flag=value']
    assert (
        builder.run_script('script.py', args).build_args()
        == '-S"script.py arg1 "arg with spaces" --flag=value"'
    )


def test_ida_command_builder_file_type():
    """Test file type option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_file_type('PE').build_args() == '-TPE'

    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_file_type('ZIP', 'classes.dex').build_args() == '-TZIP:classes.dex'


def test_ida_command_builder_empty_database():
    """Test empty database option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.empty_database().build_args() == '-t'


def test_ida_command_builder_windows_directory():
    """Test Windows directory option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_windows_directory('C:\\Windows').build_args() == '-WC:\\Windows'


def test_ida_command_builder_no_segmentation():
    """Test no segmentation option"""
    builder = ida_domain.IdaCommandBuilder()
    assert builder.no_segmentation().build_args() == '-x'


def test_ida_command_builder_debug_flags():
    """Test debug flags option"""
    # Test with numeric flags
    builder = ida_domain.IdaCommandBuilder()
    assert builder.set_debug_flags(0x404).build_args() == '-z404'

    # Test with named flags
    builder = ida_domain.IdaCommandBuilder()
    flags = ['flirt', 'type_system']
    assert builder.set_debug_flags(flags).build_args() == '-z4004'


def test_ida_command_builder_text_mode():
    """Test text mode option (doesn't affect args)"""
    builder = ida_domain.IdaCommandBuilder()
    builder.use_text_mode()
    assert builder.build_args() == ''


def test_ida_command_builder_method_chaining():
    """Test method chaining"""
    builder = ida_domain.IdaCommandBuilder()
    args = (
        builder.auto_analysis(False).set_log_file('analysis.log').set_processor('arm').build_args()
    )
    assert args == '-a -Lanalysis.log -parm'


def test_ida_command_builder_complex_scenario1():
    """Test complex scenario 1"""
    builder = ida_domain.IdaCommandBuilder()
    args = (
        builder.autonomous(True)
        .new_database(True)
        .set_compiler('gcc', 'x64')
        .set_processor('arm')
        .run_script('analyze.py', ['deep', '--verbose'])
        .build_args()
    )
    assert args == '-A -c -Cgcc:x64 -parm -S"analyze.py deep --verbose"'


def test_ida_command_builder_complex_scenario2():
    """Test complex scenario 2"""
    builder = ida_domain.IdaCommandBuilder()
    args = (
        builder.set_output_database('project.idb')
        .compress_database()
        .set_file_type('ZIP', 'classes.dex')
        .set_debug_flags(0x10004)  # debugger + flirt
        .build_args()
    )
    assert args == '-c -oproject.idb -P+ -TZIP:classes.dex -z10004'


def test_ida_command_builder_getters():
    """Test getter methods that are available"""
    builder = ida_domain.IdaCommandBuilder()

    assert builder._auto_analysis
    builder.auto_analysis(False)
    assert not builder._auto_analysis


def test_api_examples(test_env):
    examples = [
        'analyze_functions.py',
        'analyze_strings.py',
        'analyze_types.py',
        'explore_database.py',
        'traverse.py',
        'flirt.py',
    ]
    for example in examples:
        script_path = Path(__file__).parent.parent / 'examples' / example
        cmd = [sys.executable, str(script_path), '-f', str(idb_path)]

        result = subprocess.run(cmd, capture_output=True, text=True)

        print(f'Example {script_path} outputs')
        print('\n[STDOUT]')
        print(result.stdout)
        print('[STDERR]')
        print(result.stderr)

        assert result.returncode == 0, f'Example {script_path} failed to run'

    # analyze_xrefs.py requires additional arguments
    script_path = Path(__file__).parent.parent / 'examples' / 'analyze_xrefs.py'
    cmd = [sys.executable, str(script_path), '-f', str(idb_path), '-a', '0xd6']

    result = subprocess.run(cmd, capture_output=True, text=True)

    print(f'Example {script_path} outputs')
    print('\n[STDOUT]')
    print(result.stdout)
    print('[STDERR]')
    print(result.stderr)

    assert result.returncode == 0, f'Example {script_path} failed to run'
